% DP832 calibration script using a 34461A or DM3068
% v1.1 - 15-Feb-2015
%
%  LaurentR (EEVblog) 2015
% Portions  Agilent Technologies 2013 (See Matlab examples)
% http://www.keysight.com/main/techSupport.jspx?cc=US&lc=eng&nid=-536902435.1058513&pid=2270273&pageMode=OV
%
% Free for any use with no warranty, obligation or liability implied by 
% LaurentR or Agilent Technologies.
%
% All credits go to the work of TooOldForThis and ted572 (EEVblog)
% 
% Their work, the latest version of this script and comments can be found
% on the EEVblog DP832 thread:
% http://www.eevblog.com/forum/testgear/rigol-dp832-firmware-updates-and-bug-list/
%
% Release notes
% v1.0 (14-Jan-2015): - Initial release
% v1.1 (15-Feb-2015): - Various cleanup
%                     - Changed the date string function for older Matlab
%                       compatibility (thanks to EEVblog's Sparky).
%                     - Tentative Rigol DM3068 support using the suggestion
%                       by EEVblog's smgvbest to set CMDSet to AGILENT (untested)

% Usage notes
% - Use at your own risk.
%   - Understand the script before using it.
%   - Do a dry run (default option) before actually storing calibration
% 
% - By default the script DOES NOT STORE CALIBRATION at the end.
%   - Once you are confident with the script and procedure, change the
%   simOnly variable to false.
%
% - A substantial warmup (>1h) of both devices is higly recommended.
%   - The DP832 temperature drift is noticeable, so calibrate at your usual
%   ambient temperature for best results.
%
% - This script requires MATLAB with the Instrument Control toolbox
% installed
%   - Tested with R2014b on Windows 7 64b
%
% - This script supports only the Rigol DP832.
%   - Tested with FW 00.01.13, NI VISA drivers and a USB connection
%   - This script can probably be used for other models, but changes are
%   likely required (e.g. to the calibration tables)
%
% - This script assumes an Agilent 34461A is used.
%   - Tested with FW 01.10, NI VISA drivers and a USB connection
%   - The 10A current range (and connector) are used for all current
%   measurements. There is a minor loss of accuracy, but it is simpler and
%   safer that way (and well within the DP832 accuracy specification).
%   Better current accuracy may be obtained by modifying the script to use
%   the 3A connector and Auto ranging to get the best accuracy as long
%   as the current is <3A.
%   - A 34401A should be able to use the script almost directly, but the
%   current calibration include a 3.2A step which is above the supported
%   range of the 34401A, so creativity is required.
%
% - Tentative support for the Rigol DM3068 is added in v1.1
%   - The support is entirely untested as I do not have a DM3068.
%   - Please report any success or failure to the EEVblog thread.
%   - Support is done using the CMDSet AGILENT function of the DM3068 which
%     supports all common 34401A voltage and current commands used in this
%     script.
%   - A basic check of the DM3068 Programming Guide has been done to ifdef
%     out any 34461A-only command (mostly temp and uptime commands).
%
% - The script assumes NI Visa drivers and 'DP832' and '34461A' Visa
% aliases for the 2 SCPI devices. Change as needed.
%
% - The script uses a pause of 2 seconds bewteen PSU commands and DMM
% readout. AFAICT, this is quite conservative. Current measurements may
% benefit from a higher pause time to allow more time for self-heating.
%
% - The script uses a sinhle 100PLC readout on the DMM. AFAICT, this is
% sufficient as the PSU is very stable. Multiple readouts with further
% averaging is possible.

% MAIN

function DP832Calib

    % Constants subject to change
    % Review/modify before using
    myPause = 2; % 2 second pause between sending value to PSU and reading DMM value
    myNplc = 100; % Max precision
    myChannels = 1:3; % Can put a smaller number for a dry run
    simOnly = true; % Set to false to actually store calibration at the end
    psuAddr = 'DP832'; % Set to the PSU Visa address or alias
    psuDrv = 'ni'; % Set to 'agilent' if using the Agilent Visa drivers for the PSU
    dmmAddr = '34461A'; % Set to the DMM Visa address or alias
    dmmDrv = 'ni'; % Set to 'agilent' if using the Agilent Visa drivers for the DMM

    % Calibration tables
    DACV12 = [0.2 0.5 1.2 2 3.2 4.1 5.2 6.9 7.5 8.7 10.1 11.8 12.6 13.5 15 15.8 16.5 17.3 18.5 19.1 19.9 20.2 20.8 21.8 22.4 22.7 23.9 24.3 25.7 26.9 27.9 28.5 28.9 29.8 30.2 32];
    ADCV12 = [0 0.05 0.1 0.5 1 5 10 12.8 20 30 32];
    DACI12 = [0.1 0.25 0.5 0.8 1 1.25 1.5 1.75 1.9 2.15 2.35 2.5 2.75 3 3.2];
    ADCI12 = [0 0.01 0.1 1 2 3 3.2];
    DACV3 = [0.1 0.2 0.4 0.85 1.2 1.8 2.55 3.1 3.4 4.1 4.5 5 5.3];
    ADCV3 = [0 0.005 0.01 0.02 0.05 0.1 0.5 1 3 5 5.3];
    DACI3 = [0.1 0.5 1 1.25 1.5 1.75 2 2.25 2.5 2.75 3 3.2];
    ADCI3 = [0 0.1 1 2 3 3.2];

    newobjs = instrfind;
    if isempty(newobjs) == 0
        fclose(newobjs);
        delete(newobjs);
        clear newobjs
    end
    try
        if simOnly
            fprintf('NOTE: This script will NOT store the calibration into the PSU at the end.\n');
        else
            fprintf('NOTE: This script WILL STORE the calibration into the PSU at the end.\n');
        end
        fprintf('If this is not your intent, please abort and modify the value of the  "simOnly" variable.\n\n');
        myPsu = visa(psuDrv, psuAddr);
        fopen(myPsu);
        psuIdn = query(myPsu, '*IDN?');
        CheckVisaError(myPsu);
        fprintf('PSU: %s\n\n', strtrim(psuIdn));
        if isempty(strfind(psuIdn, 'DP832'))
            err = MException('Calib:BadInput', 'ERROR: Unsupported PSU');
            throw(err);
        end
        fprintf(myPsu, '*RST');   
        CheckVisaError(myPsu);
        
        myDmm = visa(dmmDrv, dmmAddr);
        fopen(myDmm);
        dmmIdn = query(myDmm, '*IDN?');
        CheckVisaError(myDmm);
        fprintf('DMM: %s\n\n', strtrim(dmmIdn));
        is34461A = ~isempty(strfind(dmmIdn, '34461A'));
        isDM3068 = ~isempty(strfind(psuIdn, 'DM3068'));
        if ~is34461A && ~isDM3068
            err = MException('Calib:BadInput', 'ERROR: Unsupported DMM');
            throw(err);
        end
        fprintf(myDmm, '*RST');
        CheckVisaError(myDmm);
        if isDM3068
            fprintf(myDmm, 'CMDSet AGILENT');
            CheckVisaError(myDmm);
        end
        
        fprintf('Remove all leads from DMM and PSU to run self-tests\n');
        input('Press Enter\n');

        fprintf('Running PSU board self-test...\n');
        r = query(myPsu, 'SYSTem:SELF:TEST:BOARD?');
        CheckVisaError(myPsu);
        fprintf('PSU self-test result: %s\n', strtrim(r));
        r = str2double(query(myPsu, 'SYSTem:SELF:TEST:TEMP?'));
        CheckVisaError(myPsu);
        fprintf('PSU internal temp: %4.1fC\n\n', r);
                
        fprintf('Running DMM self-test...\n');
        r = query(myDmm, '*TST?');
        CheckVisaError(myDmm);
        fprintf('DMM self-test result: %s (0 = PASS, 1=FAIL)\n', strtrim(r));
        if is34461A
            r = str2double(query(myDmm, 'SYST:TEMP?'));
            CheckVisaError(myDmm);
            fprintf('DMM internal temp: %4.1fC\n', r);
            upt = query(myDmm, 'SYST:UPT?');
            CheckVisaError(myDmm);
            uptn = sscanf(upt, '%d,%d,%d,%d');
            fprintf('DMM uptime: %dd %dh %dm %ds\n', uptn(1), uptn(2), uptn(3), uptn(4));
        end

        for ch = myChannels
            if (ch==1) || (ch==2)
                DACV = DACV12;
                ADCV = ADCV12;
                DACI = DACI12;
                ADCI = ADCI12;
            else
                DACV = DACV3;
                ADCV = ADCV3;
                DACI = DACI3;
                ADCI = ADCI3;
            end

            fprintf('\nStarting channel %d\n\n', ch);
            fprintf(myPsu, 'CALibration:Start 11111,CH%d', ch);
            fprintf(myPsu, 'CALibration:Clear CH%d,ALL', ch);
            fprintf(myPsu, '*RST');
            CheckVisaError(myPsu);

            fprintf('Starting Voltage: connect the DMM VOLTAGE inputs to PSU channel %d\n', ch);
            input('Press Enter');

            fprintf(myDmm, 'CONF:VOLT:DC AUTO');
            fprintf(myDmm, 'VOLT:DC:NPLC %d', myNplc);
            CheckVisaError(myDmm);

            fprintf(myPsu, 'OUTPUT CH%d,ON', ch);
            CheckVisaError(myPsu);

            fprintf('Running DAC-V for channel %d\n', ch);
            for i = 1:size(DACV, 2)
                myStep = i-1;
                fprintf(myPsu, sprintf('CALibration:Set CH%d,V,%d,%fV,1', ch, myStep, DACV(i)));
                CheckVisaError(myPsu);
                pause(myPause);
                DmmV = str2double(query(myDmm,'READ?'));
                CheckVisaError(myDmm);
                fprintf(myPsu, sprintf('CALibration:MEAS CH%d,V,%d,%fV,1', ch, myStep, DmmV));
                CheckVisaError(myPsu);
                fprintf('DACV Ch:%d - Step:%d - Nom:%fV - DMM:%fV\n', ch, myStep, DACV(i), DmmV);
            end

            fprintf('Running ADC-V for channel %d\n', ch);
            for i = 1:size(ADCV, 2)
                myStep = i-1;
                fprintf(myPsu, sprintf('CALibration:Set CH%d,V,%d,%fV,0', ch, myStep, ADCV(i)));
                CheckVisaError(myPsu);
                pause(myPause);
                DmmV = str2double(query(myDmm,'READ?'));
                CheckVisaError(myDmm);
                fprintf(myPsu, sprintf('CALibration:MEAS CH%d,V,%d,%fV,0', ch, myStep, DmmV));
                CheckVisaError(myPsu);
                fprintf('ADCV Ch:%d - Step:%d - Nom:%fV - DMM:%fV\n', ch, myStep, ADCV(i), DmmV);
            end

            fprintf(myPsu, 'OUTPUT CH%d,OFF', ch);
            CheckVisaError(myPsu);

            fprintf('Starting Current: connect the DMM 10A CURRENT inputs to PSU channel %d\n', ch);
            input('Press Enter');

            fprintf(myDmm, 'CONF:CURR:DC 10A'); %Sets input to 10A port
            fprintf(myDmm, 'CURR:DC:NPLC %d', myNplc); 
            CheckVisaError(myDmm);

            fprintf(myPsu, 'OUTPUT CH%d,ON', ch);
            CheckVisaError(myPsu);

            fprintf('Running DAC-I for channel %d\n', ch);
            for i = 1:size(DACI, 2)
                myStep = i-1;
                fprintf(myPsu, sprintf('CALibration:Set CH%d,C,%d,%fA,1', ch, myStep, DACI(i)));
                CheckVisaError(myPsu);
                pause(myPause);
                DmmI = str2double(query(myDmm,'READ?'));
                CheckVisaError(myDmm);
                fprintf(myPsu, sprintf('CALibration:MEAS CH%d,C,%d,%fA,1', ch, myStep, DmmI));
                CheckVisaError(myPsu);
                fprintf('DACI Ch:%d - Step:%d - Nom:%fA - DMM:%fA\n', ch, myStep, DACI(i), DmmI);
            end

            fprintf('Running ADC-I for channel %d\n', ch);
            for i = 1:size(ADCI, 2)
                myStep = i-1;
                fprintf(myPsu, sprintf('CALibration:Set CH%d,C,%d,%fA,0', ch, myStep, ADCI(i)));
                CheckVisaError(myPsu);
                pause(myPause);
                DmmI = str2double(query(myDmm,'READ?'));
                CheckVisaError(myDmm);
                fprintf(myPsu, sprintf('CALibration:MEAS CH%d,C,%d,%fA,0', ch, myStep, DmmI));
                CheckVisaError(myPsu);
                fprintf('ADCI Ch:%d - Step:%d - Nom:%fA - DMM:%fA\n', ch, myStep, ADCI(i), DmmI);
            end

            fprintf(myPsu, 'OUTPUT CH%d,OFF', ch);
            CheckVisaError(myPsu);
            if simOnly
                fprintf('SKIPPING STORING CALIBRATION\n');
                fprintf('Change the "simOnly" variable to false when you want to proceed with actual calibration\n');
            else
                fprintf('Writing calibration channel %d\n', ch);
                fprintf(myPsu, sprintf('CALibration:End %s,CH%d', datestr(now,'mm/dd/yyyy'), ch));
                CheckVisaError(myPsu);
            end

            fprintf('Calibration DONE channel %d\n', ch);
        end

        fprintf('\nCALIBRATION DONE\n');

    catch MExc
        MExc.message
        delete_all_objs()

    end
end

function delete_all_objs
%DELETE ALL OBJS
%Usage
%  Clean up all instrument objects not properly closed.
%  Run once at beginning of script or at the end to clean up objects
%
    newobjs = instrfind;
    if isempty(newobjs) == 0 
      % if using device objects may need to disconnect instead of close
      % disconnect(newobjs)
      fclose(newobjs);
      delete(newobjs);
      clear newobjs
    end
end

function VisaErr = CheckVisaError(myVisa)
%CHECK VISA ERROR
%Usage
%  Checks SCPI SYST:ERR? to make sure it says some version of 'No Error'.
%  If it doesn't, repeat until it does, and throw an esxception with all
%  the logged errors.
%  Works for both 34461A and DP832
%
    fprintf(myVisa, 'SYST:ERR?');
    errStr = fscanf(myVisa);

    if (strfind(errStr, 'No error')) %If no error, then return
        return;
        %If there is an error, read out all of the errors and return them in an exception
    else
        errStr2 = errStr;
        NoErr = strfind(errStr2, 'No error');
        while (isempty(NoErr));
            fprintf(myVisa, 'SYST:ERR?');
            errStr2 = fscanf(myVisa);
            NoErr = strfind(errStr2, 'No error');
            if (isempty(NoErr))
                errStr = strcat(errStr, '\n', errStr2);
            end
        end
        errStr = strcat('Exception: Encountered system error(s)\n', errStr)
        err = MException('ResultChk:BadInput', errStr);
        throw(err)
    end
end    


